package java.beans;

import sun.reflect.misc.ReflectUtil;
import java.lang.reflect.Proxy;
import java.lang.reflect.InvocationTargetException;
import java.security.PrivilegedAction;
import java.lang.reflect.Method;
import sun.reflect.misc.MethodUtil;
import java.security.AccessController;
import java.security.AccessControlContext;
import java.lang.reflect.InvocationHandler;

public class EventHandler implements InvocationHandler
{
    private Object target;
    private String action;
    private final String eventPropertyName;
    private final String listenerMethodName;
    private final AccessControlContext acc;
    
    @ConstructorProperties({ "target", "action", "eventPropertyName", "listenerMethodName" })
    public EventHandler(Object target, String action, String eventPropertyName, String listenerMethodName) {
        this.acc = AccessController.getContext();
        this.target = target;
        this.action = action;
        if (target == null) {
            throw new NullPointerException("target must be non-null");
        }
        if (action == null) {
            throw new NullPointerException("action must be non-null");
        }
        this.eventPropertyName = eventPropertyName;
        this.listenerMethodName = listenerMethodName;
        return;
    }
    
    public Object getTarget() {
        return this.target;
    }
    
    public String getAction() {
        return this.action;
    }
    
    public String getEventPropertyName() {
        return this.eventPropertyName;
    }
    
    public String getListenerMethodName() {
        return this.listenerMethodName;
    }
    
    private Object applyGetters(Object obj, String s) {
        if (s == null || s.equals("")) {
            return obj;
        }
        int endIndex = s.indexOf(46);
        if (endIndex == -1) {
            endIndex = s.length();
        }
        String substring = s.substring(0, endIndex);
        String substring2 = s.substring(Math.min(endIndex + 1, s.length()));
        try {
            Method m = null;
            if (obj != null) {
                m = Statement.getMethod(obj.getClass(), "get" + NameGenerator.capitalize(substring), new Class[0]);
                if (m == null) {
                    m = Statement.getMethod(obj.getClass(), "is" + NameGenerator.capitalize(substring), new Class[0]);
                }
                if (m == null) {
                    m = Statement.getMethod(obj.getClass(), substring, new Class[0]);
                }
            }
            if (m == null) {
                throw new RuntimeException("No method called: " + substring + " defined on " + obj);
            }
            return this.applyGetters(MethodUtil.invoke(m, obj, new Object[0]), substring2);
        }
        catch (Exception cause) {
            throw new RuntimeException("Failed to call method: " + substring + " on " + obj, cause);
        }
    }
    
    @Override
    public Object invoke(final Object o, final Method method, final Object[] array) {
        AccessControlContext acc = this.acc;
        if (acc == null && System.getSecurityManager() != null) {
            throw new SecurityException("AccessControlContext is not set");
        }
        return AccessController.doPrivileged(new PrivilegedAction<Object>() {
            @Override
            public Object run() {
                return EventHandler.access$000(EventHandler.this, o, method, array);
            }
        }, acc);
    }
    
    private Object invokeInternal(Object o, Method method, Object[] array) {
        String name = method.getName();
        if (method.getDeclaringClass() == Object.class) {
            if (name.equals("hashCode")) {
                return new Integer(System.identityHashCode(o));
            }
            if (name.equals("equals")) {
                return (o == array[0]) ? Boolean.TRUE : Boolean.FALSE;
            }
            if (name.equals("toString")) {
                return o.getClass().getName() + 64 + Integer.toHexString(o.hashCode());
            }
        }
        if (this.listenerMethodName == null || this.listenerMethodName.equals(name)) {
            Object[] params;
            Class[] array2;
            if (this.eventPropertyName == null) {
                params = new Object[0];
                array2 = new Class[0];
            }
            else {
                Object applyGetters = this.applyGetters(array[0], this.getEventPropertyName());
                params = new Object[] { applyGetters };
                array2 = new Class[] { (applyGetters == null) ? null : applyGetters.getClass() };
            }
            try {
                int lastIndex = this.action.lastIndexOf(46);
                if (lastIndex != -1) {
                    this.target = this.applyGetters(this.target, this.action.substring(0, lastIndex));
                    this.action = this.action.substring(lastIndex + 1);
                }
                Method m = Statement.getMethod(this.target.getClass(), this.action, array2);
                if (m == null) {
                    m = Statement.getMethod(this.target.getClass(), "set" + NameGenerator.capitalize(this.action), array2);
                }
                if (m == null) {
                    throw new RuntimeException("No method called " + this.action + " on " + this.target.getClass() + ((array2.length == 0) ? " with no arguments" : (" with argument " + array2[0])));
                }
                return MethodUtil.invoke(m, this.target, params);
            }
            catch (IllegalAccessException cause) {
                throw new RuntimeException(cause);
            }
            catch (InvocationTargetException ex) {
                Throwable targetException = ex.getTargetException();
                throw (targetException instanceof RuntimeException) ? ((RuntimeException)targetException) : new RuntimeException(targetException);
            }
        }
        else {
            return null;
        }
    }
    
    public static <T> T create(Class<T> clazz, Object o, String s) {
        return create(clazz, o, s, null, null);
    }
    
    public static <T> T create(Class<T> clazz, Object o, String s, String s2) {
        return create(clazz, o, s, s2, null);
    }
    
    public static <T> T create(Class<T> clazz, Object o, String s, String s2, String s3) {
        final EventHandler eventHandler = new EventHandler(o, s, s2, s3);
        if (clazz == null) {
            throw new NullPointerException("listenerInterface must be non-null");
        }
        return AccessController.doPrivileged(new PrivilegedAction<T>() {
            
            @Override
            public T run() {
                return Proxy.newProxyInstance(loader, interfaces, eventHandler);
            }
        });
    }
    
    private static ClassLoader getClassLoader(Class<?> clazz) {
        ReflectUtil.checkPackageAccess(clazz);
        ClassLoader classLoader = clazz.getClassLoader();
        if (classLoader == null) {
            classLoader = Thread.currentThread().getContextClassLoader();
            if (classLoader == null) {
                classLoader = ClassLoader.getSystemClassLoader();
            }
        }
        return classLoader;
    }
    
    static Object access$000(EventHandler eventHandler, Object o, Method method, Object[] array) {
        return eventHandler.invokeInternal(o, method, array);
    }
}
